package cn.com.qmth.examcloud.starters.face.verify.common;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.nio.file.Files;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class CommonUtils {

    private final static Logger log = LoggerFactory.getLogger(CommonUtils.class);

    public static String toBase64(File imageFile) {
        if (imageFile == null || !imageFile.exists()) {
            throw new IllegalArgumentException("文件不存在！");
        }

        String regex = "\\.(?:jpg|jpeg|png)$";
        Matcher matcher = Pattern.compile(regex).matcher(imageFile.getName().toLowerCase());
        if (!matcher.find()) {
            throw new IllegalArgumentException("文件格式不正确，仅支持（jpg、png）！");
        }

        try (InputStream is = Files.newInputStream(imageFile.toPath());) {
            byte[] bytes = IOUtils.toByteArray(is);
            return Base64.encodeBase64String(bytes);
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            throw new IllegalArgumentException(e);
        }
    }

    public static String urlEncode(String value) {
        if (value == null) {
            return "";
        }

        try {
            return URLEncoder.encode(value, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            log.error(e.getMessage(), e);
            throw new IllegalArgumentException(e);
        }
    }

}
