package cn.com.qmth.examcloud.starters.face.verify.service.impl;

import cn.com.qmth.examcloud.starters.face.verify.FaceVerifyProperties;
import cn.com.qmth.examcloud.starters.face.verify.common.CommonUtils;
import cn.com.qmth.examcloud.starters.face.verify.common.FaceResult;
import cn.com.qmth.examcloud.starters.face.verify.common.JsonHelper;
import cn.com.qmth.examcloud.starters.face.verify.service.FaceVerifyService;
import cn.com.qmth.examcloud.starters.face.verify.service.baidu.BaiduApiHelper;
import cn.com.qmth.examcloud.starters.face.verify.service.faceplus.FacePlusApiHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

public class FaceVerifyServiceImpl implements FaceVerifyService {

    private final static Logger log = LoggerFactory.getLogger(FaceVerifyServiceImpl.class);

    private FaceVerifyProperties properties;

    @Override
    public FaceResult faceVerifyByBaidu(String imageUrl) {
        List<Map<String, String>> images = new ArrayList<>();
        Map<String, String> imageData = new TreeMap<>();
        imageData.put("image_type", "URL");
        imageData.put("image", imageUrl);
        images.add(imageData);

        String params = new JsonHelper().toJson(images);
        return BaiduApiHelper.faceVerify(properties, params);
    }

    @Override
    public FaceResult faceVerifyByBaidu(File imageFile) {
        String imageBase64 = CommonUtils.toBase64(imageFile);

        List<Map<String, String>> images = new ArrayList<>();
        Map<String, String> imageData = new TreeMap<>();
        imageData.put("image_type", "BASE64");
        imageData.put("image", imageBase64);
        images.add(imageData);

        String params = new JsonHelper().toJson(images);
        if (properties.getBaiduLocalEnabled()) {
            return BaiduApiHelper.faceVerifyUseLocalApi(properties, params);
        }
        return BaiduApiHelper.faceVerify(properties, params);
    }

    @Override
    public FaceResult faceDetectByBaidu(String imageUrl) {
        // image_type: URL、BASE64、FACE_TOKEN，其中“本地化部署”不支持 URL 方式
        Map<String, String> imageData = new TreeMap<>();
        imageData.put("image_type", "URL");
        imageData.put("image", imageUrl);

        String params = new JsonHelper().toJson(imageData);
        return BaiduApiHelper.faceDetect(properties, params);
    }

    @Override
    public FaceResult faceDetectByBaidu(File imageFile) {
        String imageBase64 = CommonUtils.toBase64(imageFile);

        Map<String, String> imageData = new TreeMap<>();
        imageData.put("image_type", "BASE64");
        imageData.put("image", imageBase64);

        String params = new JsonHelper().toJson(imageData);
        if (properties.getBaiduLocalEnabled()) {
            return BaiduApiHelper.faceDetectUseLocalApi(properties, params);
        }
        return BaiduApiHelper.faceDetect(properties, params);
    }

    @Override
    public FaceResult faceCompareByBaidu(String imageUrl, String imageUrl2) {
        List<Map<String, String>> images = new ArrayList<>();
        Map<String, String> imageData = new TreeMap<>();
        imageData.put("image_type", "URL");
        imageData.put("image", imageUrl);
        images.add(imageData);

        Map<String, String> imageData2 = new TreeMap<>();
        imageData2.put("image_type", "URL");
        imageData2.put("image", imageUrl2);
        images.add(imageData2);

        String params = new JsonHelper().toJson(images);
        return BaiduApiHelper.faceCompare(properties, params);
    }

    @Override
    public FaceResult faceCompareByBaidu(File imageFile, File imageFile2) {
        String imageBase64 = CommonUtils.toBase64(imageFile);
        String imageBase64_2 = CommonUtils.toBase64(imageFile2);

        List<Map<String, String>> images = new ArrayList<>();
        Map<String, String> imageData = new TreeMap<>();
        imageData.put("image_type", "BASE64");
        imageData.put("image", imageBase64);
        images.add(imageData);

        Map<String, String> imageData2 = new TreeMap<>();
        imageData2.put("image_type", "BASE64");
        imageData2.put("image", imageBase64_2);
        images.add(imageData2);

        String params = new JsonHelper().toJson(images);
        if (properties.getBaiduLocalEnabled()) {
            return BaiduApiHelper.faceCompareUseLocalApi(properties, params);
        }
        return BaiduApiHelper.faceCompare(properties, params);
    }

    @Override
    public FaceResult faceDetectByFacePlus(String imageUrl) {
        // 参数：image_url、image_base64、image_file
        Map<String, String> params = new TreeMap<>();
        params.put("image_url", imageUrl);

        return FacePlusApiHelper.faceDetect(properties, params);
    }

    @Override
    public FaceResult faceDetectByFacePlus(File imageFile) {
        String imageBase64 = CommonUtils.toBase64(imageFile);

        Map<String, String> params = new TreeMap<>();
        params.put("image_base64", imageBase64);

        return FacePlusApiHelper.faceDetect(properties, params);
    }

    @Override
    public FaceResult faceCompareByFacePlus(String imageUrl, String imageUrl2) {
        // 参数：image_url、image_base64、image_file、face_token
        Map<String, String> params = new TreeMap<>();
        params.put("image_url1", imageUrl);
        params.put("image_url2", imageUrl2);

        return FacePlusApiHelper.faceCompare(properties, params);
    }

    @Override
    public FaceResult faceCompareByFacePlus(File imageFile, File imageFile2) {
        String imageBase64 = CommonUtils.toBase64(imageFile);
        String imageBase64_2 = CommonUtils.toBase64(imageFile2);

        Map<String, String> params = new TreeMap<>();
        params.put("image_base64_1", imageBase64);
        params.put("image_base64_2", imageBase64_2);

        return FacePlusApiHelper.faceCompare(properties, params);
    }

    public void setProperties(FaceVerifyProperties properties) {
        this.properties = properties;
    }

}
